import pymc as pm
import kabuki
from kabuki import Parameter

class Regression(kabuki.Hierarchical):
    def __init__(self, data, **kwargs):
        super(Regression, self).__init__(data, **kwargs)

        self.params = [Parameter('beta', lower=-100, upper=100, init=0),
                       Parameter('sigma', lower=1e-8, upper=100, init=.1),
                       Parameter('like', is_bottom_node=True)]

    def get_bottom_node(self, param, params):
        if param.name == 'like':
            # Create likelihood
            return pm.Normal(param.full_name,
                             mu=params['mu'],
                             tau=params['sigma']**-2,
                             value=param.data['value'],
                             observed=True)

        else:
            print "Not found."



import unittests
import kabuki
import kabuki.regression
import numpy as np
import pandas as pd

class TestRegression(unittests.UnitTest):
    def __init__(self, *args, **kwargs):
        super(TestRegression, self).__init__(*args, **kwargs)

        # Generate data
        subjs = 10
        samples_per_group = 200
        group_mean = 0.
        group_sigma = 1.
        group_beta = 1.

        #index = pd.MultiIndex()
        samples = pd.DataFrame(subjs*samples_per_group, {'subj_idx': int, 'value': np.float32})
        for subj in range(subjs):





# Sample configurable:
from IPython.config.configurable import Configurable
from IPython.utils.traitlets import *

class Model(Configurable):
    name = Unicode(u'defaultname', config=True)
    model_type = String('kabuki.Regression')
    data = String('data.csv')
    depends_on = Dict()
    kwargs = Dict()
    samples = Int(5000)
    burn = Int(1000)
    thin = Int(3)




#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import numpy.lib.recfunctions as rec
import pymc as pm
from copy import copy

import kabuki

#@hierarchical
class Prototype(kabuki.Hierarchical):
    def __init__(self, data, **kwargs):
        self.data = data

    def get_root_param(self, param, tag, pos=None):
        return None

    def get_tau_param(self, param, tag):
        return None
    
    def get_subj_param(self, param_name, parent_mean, parent_tau, subj_idx, pos=None):
        return pm.Normal('%s%i'%(param_name, subj_idx), mu=parent_mean, tau=parent_tau)


    
#@kabuki.hierarchical
class Regression(Prototype):
    param_names = ('theta', 'x')
    
    def get_root_param(self, param, tag):
        return pm.Uniform('%s%s'%(param,tag), lower=0, upper=50)

    def get_tau_param(self, param, tag):
        return None
    
    def get_observed(self, name, subj_data, params, idx=None):
        @deterministic(plot=False)
        def modelled_y(x=params['x'], theta=params['theta']):
            """Return y computed from the straight line model, given the
            inferred true inputs and the model paramters."""
            slope, intercept = theta
            return slope*x + intercept

        return (pm.Normal(name, mu=modelled_y, tau=2, value=subj_data['dependent'], observed=True), modelled_y)

#@kabuki.hierarchical
class ANOVA(Prototype):
    param_names = ('base', 'effect')
    
    def get_root_param(self, param, all_params, tag, pos=None):
        if pos is not None:
            # Check if last element
            if pos[0] == pos[1]:
                param_full_name = '%s%s'%(param,tag)
                # Set parameter so that they sum to zero
                args = tuple([all_params[p] for p in all_params if p.startswith(param)])
                return pm.Deterministic(kabuki.utils.neg_sum,
                                        param_full_name,
                                        param_full_name,
                                        parents={'args':args})

        return pm.Uniform('%s%s'%(param,tag), lower=-5, upper=5)

    def get_tau_param(self, param, all_params, tag):
        return pm.Uniform('%s%s'%(param,tag), lower=0, upper=800, plot=False)
    
    def get_subj_param(self, param_name, parent_mean, parent_tau, subj_idx, all_params, tag, pos=None):
        param_full_name = '%s%s%i'%(param_name, tag, subj_idx)
        if pos is not None:
            if pos[0] == pos[1]:
                # Set parameter so that they sum to zero
                print all_params
                args = tuple([all_params[p][subj_idx] for p in all_params if p.startswith(param_name) and all_params[p][subj_idx] is not None])
                return pm.Deterministic(kabuki.utils.neg_sum,
                                        param_full_name,
                                        param_full_name,
                                        parents={'args':args})
                                        
        return pm.Normal(param_full_name, mu=parent_mean, tau=parent_tau, plot=False)

    def get_observed(self, name, subj_data, params, idx=None):
        return pm.Normal(name, value=subj_data['score'], mu=params['base'][idx]+params['effect'][idx], tau=2,
                         observed=True, plot=False)
    



from __future__ import division
import numpy as np
import pymc as pm
import re
from matplotlib.pylab import figure
import matplotlib.pyplot as plt
import sys, os
import scipy as sc

try:
    import pandas as pd
except:
    print "Pandas not found, posterior predictive checks won't work."

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

def convert_model_to_dictionary(model):
    """convert_model_to_dictionary(model)
    transform a set or list of nodes to a dictionary
    """
    d = {}
    for node in model:
        d[node.__name__] = node
    return d

def get_group_nodes(nodes, return_list=False):
    """
    get_group_nodes(model)
    get only the group nodes from the model
    """

    if type(nodes) is dict:
        group_nodes = {}
        for name, node in nodes.iteritems():
            if (re.search('[A-Za-z)][0-9]+$',name) == None) and \
               not name.startswith('Metropolis') and \
               not name.startswith('deviance'):
                group_nodes[name] = node
        if return_list:
            return group_nodes.values()
        else:
            return group_nodes
    else:
        root = [z for z in nodes if re.search('[A-Za-z)][0-9]+$',z.__name__) == None]
        return root

def get_subjs_numbers(mc):
    if isinstance(mc, pm.MCMC):
        nodes = mc.stochastics
    else:
        nodes = mc

    s = [re.search('[0-9]+$',z.__name__) for z in nodes]
    return list(set([int(x) for x in s if x != None]))

def get_subj_nodes(model, startswith=None, i_subj=None):
    """get_subj_nodes(model, i_subj=None):
    return the nodes of subj i_subj. if is_subj is None then return all subjects' node
    if i_subj is -1, return root nodes

    """
    if isinstance(model, pm.MCMC):
        nodes = model.stochastics
    else:
        nodes = model

    if startswith is None:
        startswith = ''

    if i_subj==-1:
        return get_group_nodes(nodes)
    else:
        if type(nodes) is dict:
            nodes = nodes.values()

        if i_subj is None:
            subj = [z for z in nodes if re.search(startswith+'[A-Za-z)][0-9]+$',z.__name__) != None]
        else:
            subj = [z for z in nodes if re.search(startswith+'[A-Za-z)]%d$'%i_subj,z.__name__) != None]

        if type(nodes) is dict:
            return convert_model_to_dictionary(subj)
        else:
            return subj

def print_stats(stats):
    print gen_stats(stats)

def gen_stats(stats):
    """
    print the model's stats in a pretty format
    Input:
        stats - the output of MCMC.stats()
    """
    names = sorted(stats.keys())
    len_name = max([len(x) for x in names])
    f_names  = ['mean', 'std', '2.5q', '25q', '50q', '75q', '97.5', 'mc_err']
    len_f_names = 6

    s = 'name'.center(len_name) + '  '
    for name in f_names:
        s += ' ' + name.center(len_f_names)
    s += '\n'

    for name in names:
        i_stats = stats[name]
        if not np.isscalar(i_stats['mean']):
            continue
        s += "%s: %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f\n" % \
        (name.ljust(len_name), i_stats['mean'], i_stats['standard deviation'],
         i_stats['quantiles'][2.5], i_stats['quantiles'][25],\
         i_stats['quantiles'][50], i_stats['quantiles'][75], \
         i_stats['quantiles'][97.5], i_stats['mc error'])

    return s

def print_group_stats(stats):
    print gen_group_stats(stats)

def gen_group_stats(stats):
    """
    print the model's group stats in a pretty format
    Input:
        stats - the output of MCMC.stats()
    """

    g_stats = {}
    keys = [z for z in stats.keys() if re.match('[0-9]',z[-1]) is None]
    keys.sort()
    for key in keys:
        g_stats[key] = stats[key]
    s = gen_stats(g_stats)

    return s

def plot_posterior_nodes(nodes, bins=50):
    """Plot interpolated posterior of a list of nodes.

    :Arguments:
        nodes : list of pymc.Node's
            List of pymc.Node's to plot the posterior of
        bins : int (default=50)
            How many bins to use for computing the histogram.

    """
    from kabuki.utils import interpolate_trace
    figure()
    lb = min([min(node.trace()[:]) for node in nodes])
    ub = max([max(node.trace()[:]) for node in nodes])
    x_data = np.linspace(lb, ub, 300)

    for node in nodes:
        trace = node.trace()[:]
        #hist = interpolate_trace(x_data, trace, range=(trace.min(), trace.max()), bins=bins)
        hist = interpolate_trace(x_data, trace, range=(lb, ub), bins=bins)
        plt.plot(x_data, hist, label=node.__name__, lw=2.)

    leg = plt.legend(loc='best', fancybox=True)
    leg.get_frame().set_alpha(0.5)


def group_plot(model, params_to_plot=(), n_bins=50, save_to=None):
    if isinstance(model, pm.MCMC):
        nodes = model.stochastics
    else:
        nodes = model
    db = model.mc.db

    for (param_name, param) in model.params_dict.iteritems():
        if len(params_to_plot) > 0 and  param_name not in params_to_plot:
            continue
        for (node_tag, group_node) in param.group_nodes.iteritems():
            g_node_trace = model.mc.db.trace(group_node.__name__)[:]
            subj_nodes = param.subj_nodes[node_tag]
            if subj_nodes == []:
                continue

            print "plotting %s" % group_node.__name__
            sys.stdout.flush()
            figure()
            lb = min([min(db.trace(x.__name__)[:]) for x in subj_nodes])
            lb = min(lb, min(g_node_trace))
            ub = max([max(db.trace(x.__name__)[:]) for x in subj_nodes])
            ub = max(ub, max(g_node_trace))
            x_data = np.linspace(lb, ub, n_bins)
            g_hist = np.histogram(g_node_trace,bins=n_bins, range=[lb, ub], normed=True)[0]
            plt.plot(x_data, g_hist, '--', label='group')
            for i in subj_nodes:
                g_hist = np.histogram(db.trace(i.__name__)[:],bins=n_bins, range=[lb, ub], normed=True)[0]
                plt.plot(x_data, g_hist, label=re.search('[0-9]+$',i.__name__).group())
            leg = plt.legend(loc='best', fancybox=True)
            leg.get_frame().set_alpha(0.5)
            name = group_node.__name__
            plt.title(name)
            plt.gcf().canvas.set_window_title(name)

            if save_to is not None:
                plt.savefig(os.path.join(save_to, "group_%s.png" % name))
                plt.savefig(os.path.join(save_to, "group_%s.pdf" % name))

def compare_all_pairwise(model):
    """Perform all pairwise comparisons of dependent parameter
    distributions (as indicated by depends_on).
     :Stats generated:
        * Mean difference
        * 5th and 95th percentile
    """
    from scipy.stats import scoreatpercentile
    from itertools import combinations
    print "Parameters\tMean difference\t5%\t95%"
    # Loop through dependent parameters and generate stats
    for params in model.group_nodes_dep.itervalues():
        # Loop through all pairwise combinations
        for p0,p1 in combinations(params, 2):
            diff = model.group_nodes[p0].trace() - model.group_nodes[p1].trace()
            perc_5 = scoreatpercentile(diff, 5)
            perc_95 = scoreatpercentile(diff, 95)
            print "%s vs %s\t%.3f\t%.3f\t%.3f" %(p0, p1, np.mean(diff), perc_5, perc_95)


def plot_all_pairwise(model):
    """Plot all pairwise posteriors to find correlations."""
    import matplotlib.pyplot as plt
    import scipy as sp
    import scipy.stats
    from itertools import combinations
    #size = int(np.ceil(np.sqrt(len(data_deps))))
    fig = plt.figure()
    fig.subplots_adjust(wspace=0.4, hspace=0.4)
    # Loop through all pairwise combinations
    for i, (p0, p1) in enumerate(combinations(model.group_nodes.values())):
        fig.add_subplot(6,6,i+1)
        plt.plot(p0.trace(), p1.trace(), '.')
        (a_s, b_s, r, tt, stderr) = sp.stats.linregress(p0.trace(), p1.trace())
        reg = sp.polyval((a_s, b_s), (np.min(p0.trace()), np.max(p0.trace())))
        plt.plot((np.min(p0.trace()), np.max(p0.trace())), reg, '-')
        plt.xlabel(p0.__name__)
        plt.ylabel(p1.__name__)

    plt.draw()

def savage_dickey(pos, post_trace, range=(-.3,.3), bins=40, prior_trace=None, prior_y=None):
    """Calculate Savage-Dickey density ratio test, see Wagenmakers et
    al. 2010 at http://dx.doi.org/10.1016/j.cogpsych.2009.12.001

    :Arguments:
        pos : float
            position at which to calculate the savage dickey ratio at (i.e. the spec hypothesis you want to test)
        post_trace : numpy.array
            trace of the posterior distribution

    :Optional:
         prior_trace : numpy.array
             trace of the prior distribution
         prior_y : numpy.array
             prior density pos
         range : (int,int)
             Range over which to interpolate and plot
         bins : int
             Over how many bins to compute the histogram over

    :Note: Supply either prior_trace or prior_y.

    """

    x = np.linspace(range[0], range[1], bins)

    if prior_trace is not None:
        # Prior is provided as a trace -> histogram + interpolate
        prior_pos = interpolate_trace(pos, prior_trace, range=range, bins=bins)

    elif prior_y is not None:
        # Prior is provided as a density for each point -> interpolate to retrieve positional density
        import scipy.interpolate
        prior_pos = prior_y #scipy.interpolate.InterpolatedUnivariateSpline(x, prior_y)(pos)
    else:
        assert ValueError, "Supply either prior_trace or prior_y keyword arguments"

    # Histogram and interpolate posterior trace at SD position
    posterior_pos = interpolate_trace(pos, post_trace, range=range, bins=bins)

    # Calculate Savage-Dickey density ratio at pos
    sav_dick = prior_pos / posterior_pos

    return sav_dick

def R_hat(samples):
    n, num_chains = samples.shape # n=num_samples
    chain_means = np.mean(samples, axis=1)
    # Calculate between-sequence variance
    between_var = n * np.var(chain_means, ddof=1)

    chain_var = np.var(samples, axis=1, ddof=1)
    within_var = np.mean(chain_var)

    marg_post_var = ((n-1.)/n) * within_var + (1./n) * between_var # 11.2
    R_hat_sqrt = np.sqrt(marg_post_var/within_var)

    return R_hat_sqrt

def test_chain_convergance(models):
    # Calculate R statistic to check for chain convergance (Gelman at al 2004, 11.4)
    params = models[0].group_params
    R_hat_param = {}
    for param_name in params.iterkeys():
        # Calculate mean for each chain
        num_samples = models[0].group_params[param_name].trace().shape[0] # samples
        num_chains = len(models)
        samples = np.empty((num_chains, num_samples))
        for i,model in enumerate(models):
            samples[i,:] = model.group_params[param_name].trace()

        R_hat_param[param_name] = R_hat(samples)

    return R_hat_param

def check_geweke(model, assert_=True):
    # Test for convergence using geweke method
    for param in model.group_params.itervalues():
        geweke = np.array(pm.geweke(param))
        if assert_:
            assert (np.any(np.abs(geweke[:,1]) < 2)), 'Chain of %s not properly converged'%param
            return False
        else:
            if np.any(np.abs(geweke[:,1]) > 2):
                print "Chain of %s not properly converged" % param
                return False

    return True

def group_cond_diff(hm, node, cond1, cond2, threshold=0):
    """
    compute the difference between different condition in a group analysis.
    The function compute for each subject the difference between 'node' under
    condition 'cond1' to 'node' under condition 'cond2'.
    by assuming that each of the differences is normal distributed
    we can easily compute the group mean and group variance of the difference.
    Then the difference is compared to 'threshold' to compute the mass of the
    group pdf which is smaller than 'threshold'

    Input:
        hm - hierachical model
        node - name of node to be analyized
        cond1 - name of condition 1
        cond2 - name of condition 2
        threshold - see description

    Output:
        group_mean - group mean of the differnce
        group_var - group variance of the difference
        mass_under_threshold  - the mass of the group pdf which is smaller than threshold
    """
    name = node
    node_dict = hm.params_include[name].subj_nodes
    n_subjs = hm._num_subjs
    subj_diff = [None]*n_subjs
    #loop over subjs
    subj_diff_mean = np.zeros(n_subjs)
    subj_diff_std = np.zeros(n_subjs)
    for i_subj in range(n_subjs):
        #compute diffrence of traces
        name1 = node_dict[cond1][i_subj].__name__
        name2 = node_dict[cond2][i_subj].__name__
        trace1 = hm.mc.db.trace(name1)[:]
        trace2 = hm.mc.db.trace(name2)[:]
        diff_trace = trace1 - trace2

        #compute stats
        subj_diff_mean[i_subj] = np.mean(diff_trace)
        subj_diff_std[i_subj]= np.std(diff_trace)

    pooled_var = 1. / sum(1. / (subj_diff_std**2))
    pooled_mean = sum(subj_diff_mean / (subj_diff_std**2)) * pooled_var

    mass_under = sc.stats.norm.cdf(threshold,pooled_mean, np.sqrt(pooled_var))

    return pooled_mean, pooled_var, mass_under

def get_traces(model):
    """Returns recarray of all traces in the model.

    :Arguments:
        model : kabuki.Hierarchical submodel or pymc.MCMC model

    :Returns:
        trace_array : recarray

    """
    if isinstance(model, pm.MCMC):
        m = model
    else:
        m = model.mc

    nodes = list(m.stochastics)

    names = [node.__name__ for node in nodes]
    dtype = [(name, np.float) for name in names]
    traces = np.empty(nodes[0].trace().shape[0], dtype=dtype)

    # Store traces in one array
    for name, node in zip(names, nodes):
        traces[name] = node.trace()[:]

    return traces

def logp_trace(model):
    """
    return a trace of logp for model
    """

    #init
    db = model.mc.db
    n_samples = db.trace('deviance').length()
    logp = np.empty(n_samples, np.double)

    #loop over all samples
    for i_sample in xrange(n_samples):
        #set the value of all stochastic to their 'i_sample' value
        for stochastic in model.mc.stochastics:
            try:
                value = db.trace(stochastic.__name__)[i_sample]
                stochastic.value = value

            except KeyError:
                print "No trace available for %s. " % stochastic.__name__

        #get logp
        logp[i_sample] = model.mc.logp

    return logp

def _evaluate_post_pred(sampled_stats, data_stats, evals=None):
    """Evaluate a summary statistics of sampled sets.

    :Arguments:
        sampled_stats : dict
            Map of summary statistic names to distributions
        data_stats : dict
            Map of summary statistic names to the data distribution

    :Returns:
        pandas.DataFrame containing the eval results as columns.
    """
    from scipy.stats import scoreatpercentile, percentileofscore
    from itertools import product

    if evals is None:
        # Generate some default evals
        evals = OrderedDict()
        evals['in credible interval'] = lambda x, y: (scoreatpercentile(x, 97.5) > y) and (scoreatpercentile(x, 2.5) < y)
        evals['quantile'] = percentileofscore
        evals['SEM'] = lambda x, y: (np.mean(x) - y)**2

    # Evalualte all eval-functions
    results = pd.DataFrame(index=sampled_stats.keys(), columns=evals.keys())
    results.index.names = ['stat']
    for stat_name in sampled_stats.iterkeys():
        for eval_name, func in evals.iteritems():
            value = func(sampled_stats[stat_name], data_stats[stat_name])
            assert np.isscalar(value), "eval function %s is not returning scalar." % eval_name
            results.ix[stat_name][eval_name] = value

    return results


def _post_pred_summary_bottom_node(bottom_node, samples=500, stats=None, plot=False, bins=100, evals=None):
    """Create posterior predictive check for a single bottom node."""
    def _calc_stats(data, stats):
        out = {}
        for name, func in stats.iteritems():
            out[name] = func(data)
        return out

    if stats is None:
        stats = OrderedDict((('mean', np.mean), ('std', np.std)))

    ############################
    # Compute stats over data
    data = bottom_node.value
    data_stats = _calc_stats(data, stats)

    ###############################################
    # Initialize posterior sample stats container
    sampled_stats = {}
    for name in stats.iterkeys():
        sampled_stats[name] = np.empty(samples)

    ##############################
    # Sample and generate stats
    for sample in range(samples):
        _parents_to_random_posterior_sample(bottom_node)
        # Generate data from bottom node
        sampled = bottom_node.random()
        sampled_stat = _calc_stats(sampled, stats)

        # Add it the results container
        for name, value in sampled_stat.iteritems():
            sampled_stats[name][sample] = value

    if plot:
        from pymc.Matplot import gof_plot
        for name, value in sampled_stats.iteritems():
            gof_plot(sampled_stats[name], data_stats[name], nbins=bins, name=name, verbose=0)

    result = _evaluate_post_pred(sampled_stats, data_stats, evals=evals)

    return result

def post_pred_check(model, samples=500, bins=100, stats=None, evals=None, plot=False):
    """Run posterior predictive check on a model.

    :Arguments:
        model : kabuki.Hierarchical
            Kabuki model over which to compute the ppc on.

    :Optional:
        samples : int
            How many samples to generate for each node.
        bins : int
            How many bins to use for computing the histogram.
        stats : dict
            User-defined statistics to compute (by default mean and std are computed)
            and evaluate over the samples.
            :Example: {'mean': np.mean, 'median': np.median}
        evals : dict
            User-defined evaluations of the statistics (by default 95 percentile and SEM).
            :Example: {'percentile': scoreatpercentile}
        plot : bool
            Whether to plot the posterior predictive distributions.

    :Returns:
        Hierarchical pandas.DataFrame with the different statistics.
    """
    print "Sampling..."
    results = []

    for name, bottom_node in model.bottom_nodes.iteritems():
        if isinstance(bottom_node, np.ndarray):
            # Group model
            results_subj = []
            subjs = []

            for i_subj, bottom_node_subj in enumerate(bottom_node):
                if bottom_node_subj is None or not hasattr(bottom_node_subj, 'random'):
                    continue # Skip non-existant nodes
                subjs.append(i_subj)
                result_subj = _post_pred_summary_bottom_node(bottom_node_subj, samples=samples, bins=bins, evals=evals, stats=stats, plot=plot)
                results_subj.append(result_subj)

            if len(results_subj) != 0:
                result = pd.concat(results_subj, keys=subjs, names=['subj'])
                results.append(result)
        else:
            # Flat model
            if bottom_node is None or not hasattr(bottom_node, 'random'):
                continue # Skip
            evals = _post_pred_summary_bottom_node(bottom_node, samples=samples, bins=bins, evals=evals, stats=stats, plot=plot)
            results.append(evals)

    return pd.concat(results, keys=model.bottom_nodes.keys(), names=['node'])

def _parents_to_random_posterior_sample(bottom_node, pos=None):
    """Walks through parents and sets them to pos sample."""
    for i, parent in enumerate(bottom_node.parents.itervalues()):
        if not isinstance(parent, pm.Node): # Skip non-stochastic nodes
            continue

        if pos is None:
            # Set to random posterior position
            pos = np.random.randint(0, len(parent.trace()))

        assert len(parent.trace()) >= pos, "pos larger than posterior sample size"
        parent.value = parent.trace()[pos]


def _post_pred_bottom_node(bottom_node, value_range, samples=10, bins=100, axis=None):
    """Calculate posterior predictive for a certain bottom node.

    :Arguments:
        bottom_node : pymc.stochastic
            Bottom node to compute posterior over.

        value_range : numpy.ndarray
            Range over which to evaluate the likelihood.

    :Optional:
        samples : int (default=10)
            Number of posterior samples to use.

        bins : int (default=100)
            Number of bins to compute histogram over.

        axis : matplotlib.axis (default=None)
            If provided, will plot into axis.
    """
    like = np.empty((samples, len(value_range)), dtype=np.float32)
    for sample in range(samples):
        _parents_to_random_posterior_sample(bottom_node)
        # Generate likelihood for parents parameters
        like[sample,:] = bottom_node.pdf(value_range)

    y = like.mean(axis=0)
    try:
        y_std = like.std(axis=0)
    except FloatingPointError:
        print "WARNING! %s threw FloatingPointError over std computation. Setting to 0 and continuing." % bottom_node.__name__
        y_std = np.zeros_like(y)

    #if len(bottom_node.value) != 0:
        # No data assigned to node
    #    return y, y_std

    #hist, ranges = np.histogram(bottom_node.value, normed=True, bins=bins)

    if axis is not None:
        # Plot pp
        axis.plot(value_range, y, label='pp', color='b')
        axis.fill_between(value_range, y-y_std, y+y_std, color='b', alpha=.8)

        # Plot data
        if len(bottom_node.value) != 0:
            axis.hist(bottom_node.value, normed=True,
                      range=(value_range[0], value_range[-1]), label='data',
                      bins=100, histtype='step', lw=2.)

        axis.set_ylim(bottom=0) # Likelihood and histogram can only be positive

    return (y, y_std) #, hist, ranges)

def plot_posterior_predictive(model, value_range=None, samples=10, columns=3, bins=100, savefig=False, prefix=None, figsize=(8,6)):
    """Plot the posterior predictive of a kabuki hierarchical model.

    :Arguments:

        model : kabuki.Hierarchical
            The (constructed and sampled) kabuki hierarchical model to
            create the posterior preditive from.

        value_range : numpy.ndarray
            Array to evaluate the likelihood over.

    :Optional:

        samples : int (default=10)
            How many posterior samples to generate the posterior predictive over.

        columns : int (default=3)
            How many columns to use for plotting the subjects.

        bins : int (default=100)
            How many bins to compute the data histogram over.

        savefig : bool (default=False)
            Whether to save the figure to a file.

        prefix : str (default=None)
            Save figure into directory prefix

    :Note:

        This function changes the current value and logp of the nodes.

    """

    if value_range is None:
        # Infer from data by finding the min and max from the nodes
        value_range = np.linspace(model.data)

    for name, bottom_node in model.bottom_nodes.iteritems():
        if isinstance(bottom_node, np.ndarray):
            if not hasattr(bottom_node[0], 'pdf'):
                continue # skip nodes that do not define pdf function
        else:
            if not hasattr(bottom_node, 'pdf'):
                continue # skip nodes that do not define pdf function

        fig = plt.figure(figsize=figsize)

        fig.suptitle(name, fontsize=12)
        fig.subplots_adjust(top=0.9, hspace=.4, wspace=.3)
        if isinstance(bottom_node, np.ndarray):
            # Group model
            for i_subj, bottom_node_subj in enumerate(bottom_node):
                if bottom_node_subj is None:
                    continue # Skip non-existant nodes
                ax = fig.add_subplot(np.ceil(len(bottom_node)/columns), columns, i_subj+1)
                ax.set_title(str(i_subj))
                _post_pred_bottom_node(bottom_node_subj, value_range,
                                       axis=ax,
                                       bins=bins)
        else:
            # Flat model
            _post_pred_bottom_node(bottom_node, value_range,
                                   axis=fig.add_subplot(111),
                                   bins=bins)

        if savefig:
            if prefix is not None:
                fig.savefig(os.path.join(prefix, name) + '.svg', format='svg')
                fig.savefig(os.path.join(prefix, name) + '.png', format='png')
            else:
                fig.savefig(name + '.svg', format='svg')
                fig.savefig(name + '.png', format='png')


def _check_bottom_node(bottom_node):
    if bottom_node is None:
        print "Bottom node is None!!"
        return False

    # Check if node has name
    if not hasattr(bottom_node, '__name__'):
        print "bottom node has missing __name__ attribute."

    name = bottom_node.__name__

    # Check if parents exist
    if not hasattr(bottom_node, 'parents'):
        print "bottom node %s is missing parents attribute." % name

    # Check if node has value
    if not hasattr(bottom_node, 'value'):
        print "bottom node %s is missing value." % name

    # Check if data is empty
    if len(bottom_node.value) == 0:
        print "values of bottom node %s is emtpy!" % name

    for i, parent in enumerate(bottom_node.parents.itervalues()):
        if not hasattr(parent, '_logp'): # Skip non-stochastic nodes
            continue


def check_model(model):
    for name, bottom_node in model.bottom_nodes.iteritems():
        if isinstance(bottom_node, np.ndarray):
            # Group model
            for i_subj, bottom_node_subj in enumerate(bottom_node):
                _check_bottom_node(bottom_node_subj)
        else:
            # Flat mode
            _check_bottom_node(bottom_node)




import kabuki
from kabuki.hierarchical import Parameter
import numpy as np
import unittest
import pymc as pm

def class_factory(num_params=1, create_group_node=True, create_subj_nodes=True):
    params_local = []
    for i_param in range(num_params):
        create_group_node = bool(round(np.random.rand()))
        if create_group_node:
            create_subj_nodes = bool(round(np.random.rand()))
        else:
            create_subj_nodes = True

        p = Parameter('test%i'%i_param, lower=1, upper=10, create_group_node=create_group_node, create_subj_nodes=True)
        params_local.append(p)

    params_local.append(Parameter('observed', is_bottom_node=True))

    class Test(kabuki.Hierarchical):
        def get_params(self):
            return params_local

        def get_bottom_node(self, param, params):
            return pm.Normal(param.full_name, mu=params['test0'], tau=1, value=param.data['score'], observed=True)

        def get_group_node(self, param):
            return pm.Uniform(param.full_name,
                              lower=param.lower,
                              upper=param.upper)


        def get_var_node(self, param):
            return pm.Uniform(param.full_name, lower=0., upper=10.,
                              value=.1, plot=self.plot_var)


        def get_subj_node(self, param):
            if param.create_group_node:
                return pm.Normal(param.full_name,
                                 mu=param.group,
                                 tau=param.var)
            else:
                return pm.Uniform(param.full_name, lower=0, upper=10)


    return Test

class TestHierarchical(unittest.TestCase):
    def runTest(self):
        pass

    def __init__(self, *args, **kwargs):
        super(TestHierarchical, self).__init__(*args, **kwargs)

        np.random.seed(31337)
        self.num_subjs = 5
        self.subjs = range(self.num_subjs)
        pts_per_subj = 100
        self.data = np.empty(self.num_subjs*pts_per_subj, dtype=([('subj_idx',np.int), ('score',np.float), ('dep', 'S8'), ('foo','S8')]))

        self.models = []
        for subj in self.subjs:
            self.data['subj_idx'][subj*pts_per_subj:(subj+1)*pts_per_subj] = subj
            self.data['score'][subj*pts_per_subj:(subj+1)*pts_per_subj] = np.random.randn(pts_per_subj)
            self.data['dep'][subj*pts_per_subj:(subj+1)*pts_per_subj] = 'dep1'
            self.data['dep'][subj*pts_per_subj+pts_per_subj/2:(subj+1)*pts_per_subj] = 'dep2'
            self.data['foo'][subj*pts_per_subj:(subj+1)*pts_per_subj] = 'bar1'
            self.data['foo'][subj*pts_per_subj+pts_per_subj/2:(subj+1)*pts_per_subj] = 'bar2'

    def setUp(self):
        for i in range(1,10):
            model_class = class_factory(num_params=i)
            self.models.append(model_class(self.data))
            self.models.append(model_class(self.data, depends_on={'test0':['dep']}))
            if i > 2:
                self.models.append(model_class(self.data, depends_on={'test0':['dep'], 'test1':['foo']}))
                self.models.append(model_class(self.data, depends_on={'test0':['dep'], 'test1':['dep','foo']}))

    def test_models(self):
        for model in self.models:
            self.tst_model(model)

    def tst_model(self, model):
        model.create_nodes()

        # Create param names that should have been created
        for param in model.params:
            print model.depends_on

            if param.name in model.depends_on.keys():
                deps = model.depends_on[param.name]
            elif param.is_bottom_node:
                vals = model.depends_on.values()
                if len(vals) > 1 and type(vals[0]) is list:
                    # Flatten
                    vals = sum(vals, [])
                deps = np.unique(np.array(vals))
            else:
                deps = []

            if len(deps)!=0:
                uniqs = np.unique(model.data[deps])
                if not param.is_bottom_node:
                    param_names = ["%s%s"%(param.name, dep) for dep in uniqs]
                else:
                    uniqs = [elem for elems in uniqs for elem in elems]
                    param_names = ["%s('%s',)"%(param.name, dep) for dep in uniqs]
            else:
                param_names = [param.name]

            # Each parameter can create multiple nodes if it depends on data
            for param_name in param_names:
                if param.create_group_node:
                    print "GROUP NODES"
                    # Check if group node exists
                    self.assertIn(param_name, model.group_nodes.keys())
                    self.assertIn(param_name+'_group', model.nodes.keys())

                    # Check node parents
                    self.assertIs(param.upper, model.group_nodes[param_name].parents['upper'])
                    self.assertIs(param.lower, model.group_nodes[param_name].parents['lower'])

                if model.is_group_model and param.create_subj_nodes and not param.is_bottom_node:
                    print "SUBJ NODES"
                    # Check if subj node exist
                    # TODO: What went wrong here?
                    #self.assertIn(param_name, model.var_nodes.keys())
                    #self.assertIn(param_name+'_var', model.nodes.keys())
                    self.assertIn(param_name, model.subj_nodes.keys())
                    self.assertIn(param_name+'_subj', model.nodes.keys())

                    # Check if subj node parents link to the correct group nodes
                    if param.create_group_node:
                        for i_subj in range(model._num_subjs):
                            self.assertIs(model.group_nodes[param_name], model.subj_nodes[param_name][i_subj].parents['mu'])
                            self.assertIs(model.var_nodes[param_name], model.subj_nodes[param_name][i_subj].parents['tau'])

                if param.is_bottom_node:
                    self.assertIn(param_name, model.bottom_nodes.keys())
                    if model.is_group_model:
                        for i_subj in range(model._num_subjs):
                            parents = model.bottom_nodes[param_name][i_subj].parents.values()
                            # Test if parents are linked to the correct subjs
                            for parent in parents:
                                if not parent == 1:
                                    self.assertIn(parent.__name__, [model.subj_nodes[x][i_subj].__name__ for x in model.subj_nodes.keys()])


# class TestBayesianANOVA(unittest.TestCase):
#     def __init__(self, *args, **kwargs):
#         return
#         super(TestBayesianANOVA, self).__init__(*args, **kwargs)

#         np.random.seed(31337)

#         params = {}
#         self.data = kabuki.utils.generate_effect_data(2, 1, .5, -1.5, 15, 50)

#         # Generate model
#         self.model = kabuki.ANOVA(data, is_subj_model=True, depends_on={'effect':['cond']})
#         self.model.mcmc(map_=False)

#     def testEstimates(self):
#         pass



import kabuki
import numpy as np
import unittest
import scipy.stats
from collections import OrderedDict
from kabuki.generate import gen_rand_data, _add_noise

normal_like = kabuki.utils.scipy_stochastic(scipy.stats.distributions.norm_gen, name='normal', longname='normal')

class TestGenerate(unittest.TestCase):
    def runTest(self):
        pass

    def test_add_noise(self):
        noise = 1
        params = OrderedDict([('loc', 0), ('scale', 1)])
        np.random.seed(31337)
        new_params = _add_noise(params, noise=noise)

        # check if copied
        self.assertFalse(new_params is params)

        # check if noise correctly added
        np.random.seed(31337)
        self.assertTrue(new_params['loc'] == np.random.normal(loc=params['loc'], scale=noise))
        self.assertTrue(new_params['scale'] == np.random.normal(loc=params['scale'], scale=noise))

        # test whether exclude works
        new_params = _add_noise(params, noise=noise, exclude_params=('scale',))
        self.assertTrue(new_params['scale'] == 1)

    def test_single_cond_no_subj(self):
        params = {'loc': 0, 'scale': 1}
        np.random.seed(31337)
        data, params_return = gen_rand_data(normal_like, params, samples=100)
        np.random.seed(31337)
        truth = np.float64(normal_like.rv.random(size=100, **params))

        np.testing.assert_array_equal(data['data'], truth)
        self.assertEqual(params, params_return)

    def test_single_cond_multi_subjs(self):
        params = OrderedDict([('loc', 0), ('scale', 1)])
        subjs = 100
        samples = 100

        # generate test data
        np.random.seed(31337)
        data, params_subjs = gen_rand_data(normal_like, params, samples=samples, subjs=subjs)

        # test subj present
        np.testing.assert_array_equal(np.unique(data['subj_idx']), range(subjs))

        # test for correct length
        np.testing.assert_array_equal(len(data), subjs*samples)

        # generate truth
        np.random.seed(31337)
        for i in range(subjs):
            new_params = _add_noise(params)
            truth = np.float64(normal_like.rv.random(size=samples, **new_params))
            np.testing.assert_array_equal(data[data['subj_idx'] == i]['data'], truth)
            self.assertEqual(params_subjs[i], new_params)

    def test_single_cond_multi_subjs_exclude(self):
        params = OrderedDict([('loc', 0), ('scale', 1)])
        subjs = 100
        samples = 100

        # generate test data
        np.random.seed(31337)
        data, params_subjs = gen_rand_data(normal_like, params, samples=samples, subjs=subjs, exclude_params=('scale',))

        # test subj present
        np.testing.assert_array_equal(np.unique(data['subj_idx']), range(subjs))

        # test for correct length
        np.testing.assert_array_equal(len(data), subjs*samples)

        # generate truth
        np.random.seed(31337)
        for i in range(subjs):
            new_params = _add_noise(params, exclude_params=('scale',))
            truth = np.float64(normal_like.rv.random(size=samples, **new_params))
            np.testing.assert_array_equal(data[data['subj_idx'] == i]['data'], truth)
            self.assertEqual(params_subjs[i], new_params)


    def test_mulltiple_cond_no_subj(self):
        samples = 100
        params = OrderedDict([('cond1', {'loc': 0, 'scale': 1}), ('cond2', {'loc': 100, 'scale': 10})])

        np.random.seed(31337)
        data, subj_params = gen_rand_data(normal_like, params, samples=samples)

        # test whether conditions are present
        np.testing.assert_array_equal(np.unique(data['condition']), ['cond1', 'cond2'])
        self.assertEqual(subj_params.keys(), ['cond1', 'cond2'])

        # test for correct length
        np.testing.assert_array_equal(len(data), 2*samples)

        # generate truth
        np.random.seed(31337)
        truth = np.float64(normal_like.rv.random(size=100, **params['cond1']))
        np.testing.assert_array_equal(data[data['condition'] == 'cond1']['data'], truth)

        truth = np.float64(normal_like.rv.random(size=100, **params['cond2']))
        np.testing.assert_array_equal(data[data['condition'] == 'cond2']['data'], truth)


    def test_column_name(self):
        params = OrderedDict([('loc', 0), ('scale', 1)])
        subjs = 100
        samples = 100

        # generate test data
        np.random.seed(31337)
        data, params_subjs = gen_rand_data(normal_like, params, samples=samples, subjs=subjs, exclude_params=('scale',), column_name='test')

        self.assertIn('test', data.dtype.names)



import numpy as np
import kabuki
import pymc as pm
import os.path
from copy import deepcopy
import matplotlib.pyplot as plt

def sort_dict(d):
    from operator import itemgetter
    return sorted(d.iteritems(), key=itemgetter(1))


def _model_type_to_class(model_type):
    if isinstance(model_type, kabuki.Hierarchical):
        model_class = model_type
    elif isinstance(model_type, str):
        model_class = kabuki.utils.find_object(model_type)
    else:
        raise TypeError('Model type %s not supported.' % type(model_type))

    return model_class


def _parse_experiment(experiment):
    data = experiment['data']
    model_type = experiment['model_type']

    if 'kwargs' in experiment:
        kwargs = deepcopy(experiment['kwargs'])
    else:
        kwargs = {}


    if 'name' in experiment:
        name = experiment['name']
    else:
        name = model_type + str(sort_dict(kwargs))

    model_class = _model_type_to_class(model_type)

    return data, model_class, kwargs, name


def run_experiment(experiment, db='sqlite', samples=10000, burn=5000, thin=3, subj_map_init=True):
    import os

    data, model_class, kwargs, name = _parse_experiment(experiment)

    if not os.path.exists(name):
        os.mkdir(name)

    m = model_class(data, **kwargs)

    if subj_map_init:
        m.subj_by_subj_map_init()

    m.mcmc(db=db, dbname=os.path.join(name, 'traces.db'))
    m.sample(samples, burn=burn, thin=thin)

    stats = kabuki.analyze.gen_stats(m.mc.stats())

    with open('%s/stats.txt'%name, 'w') as f:
        f.write("%f\n" % m.mc.dic)
        f.write(stats)

    return stats


def run_experiments(experiments, mpi=False, **kwargs):
    if mpi:
        import mpi4py_map
        results = mpi4py_map.map(run_experiment, experiments, **kwargs)
    else:
        results = [run_experiment(experiment, **kwargs) for experiment in experiments]

    return results


def load_experiment(experiment, db_loader=pm.database.sqlite.load, db_fname='traces.db'):
    experiment['model'] = load_model(experiment, db_loader=db_loader, db_fname=db_fname)
    return experiment

def load_model(experiment, db_loader=pm.database.sqlite.load, db_fname='traces.db'):
    data, model_class, kwargs, name = _parse_experiment(experiment)
    m = model_class(data, **kwargs)

    m.load_db(os.path.join(name, db_fname), db_loader=db_loader)

    return m

def load_ppc(experiment):
    import pandas as pd
    data, model_class, kwargs, name = _parse_experiment(experiment)
    post_pred = pd.read_csv(os.path.join(name, 'post_pred.csv'))

    return post_pred

def load_ppcs(experiments):
    import pandas as pd
    post_preds = []
    for experiment in experiments:
        try:
            ppc = load_ppc(experiment)
        except:
            pass
        post_preds.append(ppc)

    model_names = [_parse_experiment(experiment)[-1] for experiment in experiments]
    print model_names
    return (model_names, post_preds)
    #return pd.concat(post_preds, keys=model_names, names=['model'])

def analyze_experiment(experiment, plot_groups=True, plot_traces=True, plot_post_pred=True, ppd=True, stats=None):
    data, model_class, kwargs, name = _parse_experiment(experiment)

    if 'model' in experiment:
        # Model already loaded, use it
        model = experiment['model']
    else: # Load it
        model = load_model(experiment)

    print "Analyzing model: %s" % name
    if plot_groups:
        kabuki.analyze.group_plot(model, save_to=name)

    if plot_traces:
        model.plot_posteriors(path=name)

    if plot_post_pred:
        print "Plotting posterior predictive"
        kabuki.analyze.plot_posterior_predictive(model, np.linspace(-1.2, 1.2, 80), savefig=True, prefix=name, columns=7, figsize=(18,18))

    if ppd:
        ppd = kabuki.analyze.post_pred_check(model, stats=stats)
        print ppd
        ppd.to_csv(os.path.join(name, 'post_pred.csv'))


def analyze_experiments(experiments, mpi=False, plot_dic=True, **kwargs):
    if mpi:
        import mpi4py_map
        results = mpi4py_map.map(analyze_experiment, experiments, **kwargs)
    else:
        results = [analyze_experiment(experiment, **kwargs) for experiment in experiments]

    if plot_dic:
        dics = [experiment['model'].mc.dic for experiment in experiments]
        names = [_parse_experiment(experiment)[-1] for experiment in experiments]

        fig = plt.figure()
        x = range(len(names))
        ax = plt.bar(x, dics, align='center')
        plt.xticks(x, names)
        plt.ylabel('DIC')
        fig.autofmt_xdate()
        fig.savefig('dic.png')
        fig.savefig('dic.pdf')

    return results



 #!/usr/bin/python
from __future__ import division
from copy import copy

import numpy as np
import numpy.lib.recfunctions as rec

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

import pymc as pm
import warnings

import kabuki
from copy import copy, deepcopy
from matplotlib.mlab import rec_drop_fields


class Parameter(object):
    """Specify a parameter of a model.

    :Arguments:
        name <str>: Name of parameter.

    :Optional:
        create_group_node <bool=True>: Create group nodes for parameter.
        create_subj_nodes <bool=True>: Create subj nodes for parameter.
        is_bottom_node <bool=False>: Is node at the bottom of the hierarchy (e.g. likelihoods)
        lower <float>: Lower bound (e.g. for a uniform distribution).
        upper <float>: Upper bound (e.g. for a uniform distribution).
        init <float>: Initialize to value.
        vars <dict>: User-defined variables, can be anything you later
            want to access.
        optional <bool=False>: Only create distribution when included.
            Otherwise, set to default value (see below).
        default <float>: Default value if optional=True.
        verbose <int=0>: Verbosity.
        var_type <string>: type of the var node, can be one of ['std', 'precision', 'sample_size']
    """

    def __init__(self, name, create_group_node=True, create_subj_nodes=True,
                 is_bottom_node=False, lower=None, upper=None, init=None,
                 vars=None, default=None, optional=False, var_lower=1e-3,
                 var_upper=10, var_type='std', verbose=0):
        self.name = name
        self.create_group_node = create_group_node
        self.create_subj_nodes = create_subj_nodes
        self.is_bottom_node = is_bottom_node
        self.lower = lower
        self.upper = upper
        self.init = init
        self.vars = vars
        self.optional = optional
        self.default = default
        self.verbose = verbose
        self.var_lower = var_lower
        self.var_upper = var_upper
        self.var_type = var_type

        if self.optional and self.default is None:
            raise ValueError("Optional parameters have to have a default value.")

        if self.is_bottom_node:
            self.create_group_node = False
            self.create_subj_nodes = True

        self.group_nodes = OrderedDict()
        self.var_nodes = OrderedDict()
        self.subj_nodes = OrderedDict()
        self.bottom_nodes = OrderedDict()

        # Pointers that get overwritten
        self.group = None
        self.subj = None
        self.tag = None
        self.data = None
        self.idx = None

    def reset(self):
        self.group = None
        self.subj = None
        self.tag = None
        self.data = None
        self.idx = None

    def get_full_name(self):
        if self.idx is not None:
            if self.tag is not None:
                return '%s%s%i'%(self.name, self.tag, self.idx)
            else:
                return '%s%i'%(self.name, self.idx)
        else:
            if self.tag is not None:
                return '%s%s'%(self.name, self.tag)
            else:
                return self.name

    full_name = property(get_full_name)

    def __repr__(self):
        return object.__repr__(self).replace(' object ', " '%s' "%self.name)


class Hierarchical(object):
    """Creation of hierarchical Bayesian models in which each subject
    has a set of parameters that are constrained by a group distribution.

    :Arguments:
        data : numpy.recarray
            Input data with a row for each trial.
            Must contain the following columns:
              * 'rt': Reaction time of trial in seconds.
              * 'response': Binary response (e.g. 0->error, 1->correct)
            May contain:
              * 'subj_idx': A unique ID (int) of the subject.
              * Other user-defined columns that can be used in depends_on
                keyword.

    :Optional:
        include : tuple
            If the model has optional arguments, they
            can be included as a tuple of strings here.

        is_group_model : bool
            If True, this results in a hierarchical
            model with separate parameter distributions for each
            subject. The subject parameter distributions are
            themselves distributed according to a group parameter
            distribution.

        depends_on : dict
            Specifies which parameter depends on data
            of a column in data. For each unique element in that
            column, a separate set of parameter distributions will be
            created and applied. Multiple columns can be specified in
            a sequential container (e.g. list)

            :Example:

            >>> depends_on={'param1':['column1']}

            Suppose column1 has the elements 'element1' and
            'element2', then parameters 'param1('element1',)' and
            'param1('element2',)' will be created and the
            corresponding parameter distribution and data will be
            provided to the user-specified method get_liklihood().

        trace_subjs : bool
             Save trace for subjs (needed for many
             statistics so probably a good idea.)

        plot_var : bool
             Plot group variability parameters
             (i.e. variance of Normal distribution.)

        replace_params : list of Parameters
            User defined parameters to replace the default ones.

    :Note:
        This class must be inherited. The child class must provide
        the following functions:
            * get_group_node(param): Return group mean distribution for param.
            * get_var_node(param): Return group variability distribution for param.
            * get_subj_node(param): Return subject distribution for param.
            * get_bottom_node(param, params): Return distribution
                  for nodes at the bottom of the hierarchy param (e.g. the model
                  likelihood). params contains the associated model
                  parameters.

        In addition, the variable self.params must be defined as a
        list of Paramater().

    """

    def __init__(self, data, is_group_model=None, depends_on=None, trace_subjs=True,
                 plot_subjs=False, plot_var=False, include=(), replace_params=None):
        # Init
        self.include = set(include)

        self.nodes = {}
        self.mc = None
        self.trace_subjs = trace_subjs
        self.plot_subjs = plot_subjs
        self.plot_var = plot_var

        # Add data_idx field to data. Since we are restructuring the
        # data, this provides a means of getting the data out of
        # kabuki and sorting to according to data_idx to get the
        # original order.
        assert('data_idx' not in data.dtype.names),'A field named data_idx was found in the data file, please change it.'
        new_dtype = data.dtype.descr + [('data_idx', '<i8')]
        new_data = np.empty(data.shape, dtype=new_dtype)
        for field in data.dtype.fields:
            new_data[field] = data[field]
        new_data['data_idx'] = np.arange(len(data))
        data = new_data
        self.data = data

        if not depends_on:
            self.depends_on = {}
        else:
            # Support for supplying columns as a single string
            # -> transform to list
            for key in depends_on:
                if type(depends_on[key]) is str:
                    depends_on[key] = [depends_on[key]]
            # Check if column names exist in data
            for depend_on in depends_on.itervalues():
                for elem in depend_on:
                    if elem not in self.data.dtype.names:
                        raise KeyError, "Column named %s not found in data." % elem
            self.depends_on = depends_on
        self.depends_dict = OrderedDict()

        if is_group_model is None:
            if 'subj_idx' in data.dtype.names:
                if len(np.unique(data['subj_idx'])) != 1:
                    self.is_group_model = True
                else:
                    self.is_group_model = False
            else:
                self.is_group_model = False

        else:
            if is_group_model:
                if 'subj_idx' not in data.dtype.names:
                    raise ValueError("Group models require 'subj_idx' column in input data.")

            self.is_group_model = is_group_model

        # Should the model incorporate multiple subjects
        if self.is_group_model:
            self._subjs = np.unique(data['subj_idx'])
            self._num_subjs = self._subjs.shape[0]

        #set Parameters
        self.params = self.get_params()

        if replace_params != None:
            self.set_user_params(replace_params)

        self.params_dict = {}
        for param in self.params:
            self.params_dict[param.name] = param


    def set_user_params(self, replace_params):
        """replace parameters with user defined parameters"""
        if isinstance(replace_params, Parameter):
            replace_params = [replace_params]
        for new_param in replace_params:
            for i in range(len(self.params)):
                if self.params[i].name == new_param.name:
                    self.params[i] = new_param


    def _get_data_depend(self):
        """Partition data according to self.depends_on.

        :Returns:
            List of tuples with the data, the corresponding parameter
            distribution and the parameter name.

        """

        params = {} # use subj parameters to feed into model
        # Create new params dict and copy over nodes

        for name, param in self.params_include.iteritems():
            # Bottom nodes are created later
            if name in self.depends_on or param.is_bottom_node:
                continue
            if self.is_group_model and param.create_subj_nodes:
                params[name] = param.subj_nodes['']
            else:
                params[name] = param.group_nodes['']

        depends_on = copy(self.depends_on)

        # Make call to recursive function that does the partitioning
        data_dep = self._get_data_depend_rec(self.data, depends_on, params, [])

        return data_dep

    def _get_data_depend_rec(self, data, depends_on, params, dep_name, param=None):
        """Recursive function to partition data and params according
        to depends_on.

        """

        # Unfortunately, this function is quite complex as it
        # recursively parcels the data.

        if len(depends_on) != 0: # If depends are present
            data_params = []
            # Get first param from depends_on
            param_name = depends_on.keys()[0]
            col_name = depends_on.pop(param_name) # Take out param
            depend_elements = np.unique(data[col_name])
            # Loop through unique elements
            for depend_element in depend_elements:
                # Append dependent element name.
                dep_name.append(depend_element)
                # Extract rows containing unique element
                data_dep = data[data[col_name] == depend_element]

                # Add a key that is only the col_name that links to
                # the correct dependent nodes. This is the central
                # trick so that later on the get_bottom_node can use
                # params[col_name] and the bottm node will get linked to
                # the correct nodes automatically.
                param = self.params_include[param_name]

                # Add the node
                if self.is_group_model and param.create_subj_nodes:
                    params[param_name] = param.subj_nodes[str(depend_element)]
                else:
                    params[param_name] = param.group_nodes[str(depend_element)]
                # Recursive call with one less dependency and the selected data.
                data_param = self._get_data_depend_rec(data_dep,
                                                       depends_on=copy(depends_on),
                                                       params=copy(params),
                                                       dep_name=copy(dep_name),
                                                       param=param)
                data_params += data_param
                # Remove last item (otherwise we would always keep
                # adding the dep elems of in one column)
                dep_name.pop()
            return data_params

        else: # Data does not depend on anything (anymore)

            #create_
            if len(dep_name) != 0:
                if len(dep_name) == 1:
                    dep_name_str = str(dep_name[0])
                else:
                    dep_name_str = str(dep_name)
            else:
                dep_name_str = ''

            return [(data, params, dep_name, dep_name_str)]

    def create_nodes(self, max_retries=8):
        """Set group level distributions. One distribution for each
        parameter.

        :Arguments:
            max_retries : int
                How often to retry when model creation
                failed (due to bad starting values).

        """
        def _create():
            for name, param in self.params_include.iteritems():
                # Bottom nodes are created elsewhere
                if param.is_bottom_node:
                    continue
                # Check if parameter depends on data
                if name in self.depends_on.keys():
                    self._set_dependent_param(param)
                else:
                    self._set_independet_param(param)

            # Init bottom nodes
            for param in self.params_include.itervalues():
                if not param.is_bottom_node:
                    continue
                self._set_bottom_nodes(param, init=True)

            # Create bottom nodes
            for param in self.params_include.itervalues():
                if not param.is_bottom_node:
                    continue
                self._set_bottom_nodes(param, init=False)

        # Include all defined parameters by default.
        self.non_optional_params = [param.name for param in self.params if not param.optional]

        # Create params dictionary
        self.params_dict = OrderedDict()
        for param in self.params:
            self.params_dict[param.name] = param

        self.params_include = OrderedDict()
        for param in self.params:
            if param.name in self.include or not param.optional:
                self.params_include[param.name] = param

        for tries in range(max_retries):
            try:
                _create()
            except (pm.ZeroProbability, ValueError) as e:
                continue
            break
        else:
            print "After %f retries, still no good fit found." %(tries)
            raise e


        # Create model dictionary
        self.nodes = {}
        self.group_nodes = {}
        self.var_nodes = {}
        self.subj_nodes = {}
        self.bottom_nodes = {}

        for name, param in self.params_include.iteritems():
            for tag, node in param.group_nodes.iteritems():
                self.nodes[name+tag+'_group'] = node
                self.group_nodes[name+tag] = node
            for tag, node in param.subj_nodes.iteritems():
                self.nodes[name+tag+'_subj'] = node
                self.subj_nodes[name+tag] = node
            for tag, node in param.var_nodes.iteritems():
                self.nodes[name+tag+'_var'] = node
                self.var_nodes[name+tag] = node
            for tag, node in param.bottom_nodes.iteritems():
                self.nodes[name+tag+'_bottom'] = node
                self.bottom_nodes[name+tag] = node

        return self.nodes

    def map(self, runs=2, warn_crit=5, method='fmin_powell', **kwargs):
        """
        Find MAP and set optimized values to nodes.

        :Arguments:
            runs : int
                How many runs to make with different starting values
            warn_crit: float
                How far must the two best fitting values be apart in order to print a warning message

        :Returns:
            pymc.MAP object of model.

        :Note:
            Forwards additional keyword arguments to pymc.MAP().

        """

        from operator import attrgetter

        maps = []

        for i in range(runs):
            # (re)create nodes to get new initival values.
            #nodes are not created for the first iteration if they already exist
            if (i > 0) or (not self.nodes):
                self.create_nodes()

            m = pm.MAP(self.nodes)
            m.fit(method, **kwargs)
            print m.logp
            maps.append(m)

        # We want to use values of the best fitting model
        sorted_maps = sorted(maps, key=attrgetter('logp'))
        max_map = sorted_maps[-1]

        # If maximum logp values are not in the same range, there
        # could be a problem with the model.
        if runs >= 2:
            abs_err = np.abs(sorted_maps[-1].logp - sorted_maps[-2].logp)
            if abs_err > warn_crit:
                print "Warning! Two best fitting MAP estimates are %f apart. Consider using more runs to avoid local minima." % abs_err

        # Set values of nodes
        for name, node in max_map._dict_container.iteritems():
            if isinstance(node, pm.ArrayContainer):
                for i,subj_node in enumerate(node):
                    if isinstance(node, pm.Node) and not subj_node.observed:
                        self.nodes[name][i].value = subj_node.value
            elif isinstance(node, pm.Node) and not node.observed:
                self.nodes[name].value = node.value

        return max_map

    def mcmc(self, *args, **kwargs):
        """
        Returns pymc.MCMC object of model.

        :Note:
            Forwards arguments to pymc.MCMC().

        """

        if not self.nodes:
            self.create_nodes()

        self.mc = pm.MCMC(self.nodes, *args, **kwargs)

        return self.mc

    def sample(self, *args, **kwargs):
        """Sample from posterior.

        :Note:
            Forwards arguments to pymc.MCMC.sample().

        """

        # init mc if needed
        if self.mc == None:
            self.mcmc()

        # suppress annoying warnings
        if ('hdf5' in dir(pm.database)) and \
           isinstance(self.mc.db, pm.database.hdf5.Database):
            warnings.simplefilter('ignore', pm.database.hdf5.tables.NaturalNameWarning)

        # sample
        self.mc.sample(*args, **kwargs)

        return self.mc


    def print_group_stats(self, fname=None):
        stats_str = kabuki.analyze.gen_group_stats(self.stats())
        if fname is None:
            print stats_str
            print "DIC: %f" % self.mc.dic
            print "logp: %f" % self.mc.logp
        else:
            with open(fname) as fd:
                fd.write(stats_str)
                fd.write("DIC: %f\n" % self.mc.dic)
                fd.write("logp: %f\n" % self.mc.logp)



    def print_stats(self, fname=None):
        stats_str = kabuki.analyze.gen_stats(self.stats())
        if fname is None:
            print stats_str
            print "DIC: %f" % self.mc.dic
            print "logp: %f" % self.mc.logp
        else:
            with open(fname) as fd:
                fd.write(stats_str)
                fd.write("DIC: %f\n" % self.mc.dic)
                fd.write("logp: %f\n" % self.mc.logp)

    def _set_dependent_param(self, param):
        """Set parameter that depends on data.

        :Arguments:
            param_name : string
                Name of parameter that depends on data for
                which to set distributions.

        """

        # Get column names for provided param_name
        depends_on = self.depends_on[param.name]

        # Get unique elements from the columns
        data_dep = self.data[depends_on]
        uniq_data_dep = np.unique(data_dep)

        #update depends_dict
        self.depends_dict[param.name] = uniq_data_dep

        # Loop through unique elements
        for uniq_date in uniq_data_dep:
            # Select data
            data_dep_select = self.data[(data_dep == uniq_date)]

            # Create name for parameter
            tag = str(uniq_date)

            # Create parameter distribution from factory
            param.tag = tag
            param.data = data_dep_select
            if param.create_group_node:
                param.group_nodes[tag] = self.get_group_node(param)
            else:
                param.group_nodes[tag] = None
            param.reset()

            if self.is_group_model and param.create_subj_nodes:
                # Create appropriate subj parameter
                self._set_subj_nodes(param, tag, data_dep_select)

        return self

    def _set_independet_param(self, param):
        """Set parameter that does _not_ depend on data.

        :Arguments:
            param_name : string
                Name of parameter.

        """

        # Parameter does not depend on data
        # Set group parameter
        param.tag = ''
        if param.create_group_node:
            param.group_nodes[''] = self.get_group_node(param)
        else:
            param.group_nodes[''] = None
        param.reset()

        if self.is_group_model and param.create_subj_nodes:
            self._set_subj_nodes(param, '', self.data)

        return self

    def _set_subj_nodes(self, param, tag, data):
        """Set nodes with a parent.

        :Arguments:
            param_name : string
                Name of parameter.
            tag : string
                Element name.
            data : numpy.recarray
                Part of the data the parameter
                depends on.

        """
        # Generate subj variability parameter var
        param.tag = 'var'+tag
        param.data = data
        if param.create_group_node:
            param.var_nodes[tag] = self.get_var_node(param)
        else:
            param.var_nodes[''] = None
        param.reset()

        # Init
        param.subj_nodes[tag] = np.empty(self._num_subjs, dtype=object)
        # Create subj parameter distribution for each subject
        for subj_idx, subj in enumerate(self._subjs):
            data_subj = data[data['subj_idx']==subj]
            param.data = data_subj
            param.group = param.group_nodes[tag]
            if param.create_group_node:
                param.var = param.var_nodes[tag]
            param.tag = tag
            param.idx = subj_idx
            param.subj_nodes[tag][subj_idx] = self.get_subj_node(param)
            param.reset()

        return self

    def _set_bottom_nodes(self, param, init=False):
        """Set parameter node that has no parent.

        :Arguments:
            param_name : string
                Name of parameter.

        :Optional:
            init : bool
                Initialize parameter.

        """
        # Divide data and parameter distributions according to self.depends_on
        data_dep = self._get_data_depend()

        # Loop through parceled data and params and create an observed stochastic
        for i, (data, params_dep, dep_name_list, dep_name_str) in enumerate(data_dep):
            dep_name = dep_name_str
            if init:
                if self.is_group_model and param.create_subj_nodes:
                    param.bottom_nodes[dep_name] = np.empty(self._num_subjs, dtype=object)
                else:
                    param.bottom_nodes[dep_name] = None
            else:
                self._create_bottom_node(param, data, params_dep, dep_name, i)

        return self

    def _create_bottom_node(self, param, data, params, dep_name, idx):
        """Create parameter node object which has no parent.

        :Note:
            Called by self._set_bottom_node().

        :Arguments:
            param_name : string
                Name of parameter.
            data : numpy.recarray
                Data on which parameter depends on.
            params : list
                List of parameters the node depends on.
            dep_name : str
                Element name the node depends on.
            idx : int
                Subject index.

        """

        if self.is_group_model:
            for i, subj in enumerate(self._subjs):
                # Select data belonging to subj
                data_subj = data[data['subj_idx'] == subj]
                # Skip if subject was not tested on this condition
                if len(data_subj) == 0:
                    continue

                ########################################
                # Unfortunately, this is a little hairy since we have
                # to find the nodes of the right subject and the right
                # condition.

                # Here we'll store all nodes belonging to the subject
                selected_subj_nodes = {}
                # Find and store corresponding nodes
                for selected_param in self.params_include.itervalues():
                    # Since groupless nodes are not created in this function we
                    # have to search for the correct node and include it in
                    # the params.
                    if selected_param.is_bottom_node:
                        continue
                    if not selected_param.create_subj_nodes:
                        if selected_param.subj_nodes.has_key(dep_name):
                            selected_subj_nodes[selected_param.name] = selected_param.group_nodes[dep_name]
                        else:
                            selected_subj_nodes[selected_param.name] = params[selected_param.name]
                    else:
                        if selected_param.subj_nodes.has_key(dep_name):
                            selected_subj_nodes[selected_param.name] = selected_param.subj_nodes[dep_name][i]
                        else:
                            selected_subj_nodes[selected_param.name] = params[selected_param.name][i]

                # Set up param
                param.tag = dep_name
                param.idx = i
                param.data = data_subj
                # Call to the user-defined function!
                bottom_node = self.get_bottom_node(param, selected_subj_nodes)
                if bottom_node is not None and len(bottom_node.value) == 0:
                    print "Warning! Bottom node %s is not linked to data. Replacing with None." % param.full_name
                    param.bottom_nodes[dep_name][i] = None
                else:
                    param.bottom_nodes[dep_name][i] = bottom_node
                param.reset()
        else: # Do not use subj params, but group ones
            # Since group nodes are not created in this function we
            # have to search for the correct node and include it in
            # the params
            for selected_param in self.params_include.itervalues():
                if selected_param.subj_nodes.has_key(dep_name):
                    params[selected_param.name] = selected_param.subj_nodes[dep_name]

            if len(data) == 0:
                # If no data is present, do not create node.
                param.bottom_nodes[dep_name][i] = None
            else:
                param.tag = dep_name
                param.data = data
                # Call to user-defined function
                bottom_node = self.get_bottom_node(param, params)
                if bottom_node is not None and len(bottom_node.value) == 0:
                    print "Warning! Bottom node %s is not linked to data. Replacing with None." % param.full_name
                    param.bottom_nodes[dep_name] = None
                else:
                    param.bottom_nodes[dep_name] = bottom_node

            param.reset()

        return self


    def get_node(self, node_name, params):
        """Returns the node object with node_name from params if node
        is included in model, otherwise returns default value.

        """
        if node_name in self.include:
            return params[node_name]
        else:
            assert self.params_dict[node_name].default is not None, "Default value of not-included parameter not set."
            return self.params_dict[node_name].default


    def stats(self, *args, **kwargs):
        """
        smart call of MCMC.stats() for the model
        """
        try:
            nchains = self.mc.db.chains
        except AttributeError:
            raise ValueError("No model found.")

        #check which chain is going to be "stat"
        if 'chain' in kwargs:
            i_chain = kwargs['chain']
        else:
            i_chain = nchains

        #compute stats
        try:
            if self._stats_chain==i_chain:
                return self._stats
        except AttributeError:
            pass
        self._stats = self.mc.stats(*args, **kwargs)
        self._stats_chain = i_chain
        return self._stats



    def load_db(self, dbname, verbose=0, db_loader=None):
        """Load samples from a database created by an earlier model
        run (e.g. by calling .mcmc(dbname='test'))
        """
        if db_loader is None:
            db_loader = pm.database.sqlite.load

        # Set up model
        if not self.nodes:
            self.create_nodes()

        # Ignore annoying sqlite warnings
        warnings.simplefilter('ignore', UserWarning)

        # Open database
        db = db_loader(dbname)

        # Create mcmc instance reading from the opened database
        self.mc = pm.MCMC(self.nodes, db=db, verbose=verbose)

        # Not sure if this does anything useful, but calling for good luck
        self.mc.restore_sampler_state()

        # Take the traces from the database and feed them into our
        # distribution variables (needed for _gen_stats())
        for node in self.mc.stochastics:
            node.trace = self.mc.trace(node.__name__)

        return self

    #################################
    # Methods that can be overwritten
    #################################
    def get_group_node(self, param):
        """Create and return a uniform prior distribution for group
        parameter 'param'.

        This is used for the group distributions.

        """
        return pm.Uniform(param.full_name,
                          lower=param.lower,
                          upper=param.upper,
                          value=param.init,
                          verbose=param.verbose)

    def get_var_node(self, param):
        """Create and return a Uniform prior distribution for the
        variability parameter 'param'.

        Note, that we chose a Uniform distribution rather than the
        more common Gamma (see Gelman 2006: "Prior distributions for
        variance parameters in hierarchical models").

        This is used for the variability fo the group distribution.

        """
        return pm.Uniform(param.full_name, lower=param.var_lower, upper=param.var_upper,
                          value=.3, plot=self.plot_var)

    def get_subj_node(self, param):
        """Create and return a Truncated Normal distribution for
        'param' centered around param.group with standard deviation
        param.var and initialization value param.init.

        This is used for the individual subject distributions.

        """
        return pm.TruncatedNormal(param.full_name,
                                  a=param.lower,
                                  b=param.upper,
                                  mu=param.group,
                                  tau=param.var**-2,
                                  plot=self.plot_subjs,
                                  trace=self.trace_subjs,
                                  value=param.init)

    def init_from_existing_model(self, pre_model, step_method, **kwargs):
        """
        initialize the value and step methods of the model using an existing model
        """
        if not self.nodes:
            self.mcmc(**kwargs)
        all_nodes = list(self.mc.stochastics)
        all_pre_nodes = list(pre_model.mc.stochastics)
        assigned_values = 0
        assigned_steps = 0

        #loop over all nodes
        for i_node in range(len(all_nodes)):
            #get name of current node
            t_name = all_nodes[i_node].__name__
            #get the matched node from the pre_model
            pre_node = [x for x in all_pre_nodes if x.__name__ == t_name]
            if len(pre_node)==0:
                continue
            pre_node = pre_node[0]
            assigned_values += 1

            all_nodes[i_node].value= pre_node.value
            if step_method:
                step_pre_node = pre_model.mc.step_method_dict[pre_node][0]
                pre_sd = step_pre_node.proposal_sd * step_pre_node.adaptive_scale_factor
                if type(step_pre_node) == pm.Metropolis:
                    self.mc.use_step_method(pm.Metropolis(all_nodes[i_node],
                                                          proposal_sd = pre_sd))
                    assigned_steps += 1

        print "assigned values to %d nodes (out of %d)." % (assigned_values, len(all_nodes))
        if step_method:
            print "assigned step methods to %d (out of %d)." % (assigned_steps, len(all_nodes))

    def plot_posteriors(self, *args, **kwargs):
        pm.Matplot.plot(self.mc, *args, **kwargs)

    def plot_posterior_predictive(self, *args, **kwargs):
        kabuki.analyze.plot_posterior_predictive(self, *args, **kwargs)

    def subj_by_subj_map_init(self, runs=2, verbose=-1, **map_kwargs):
        """
        initializing nodes by finding the MAP for each subject separately
        Input:
            runs - number of MAP runs for each subject
            map_kwargs - other arguments that will be passes on to the map function

        Note: This function should be run prior to the nodes creation, i.e.
        before running mcmc() or map()
        """

        # check if nodes were created. if they were it cause problems for deepcopy
        assert (not self.nodes), "function should be used before nodes are initialized."

        # init
        subjs = self._subjs
        n_subjs = len(subjs)

        empty_s_model = deepcopy(self)
        empty_s_model.is_group_model = False
        del empty_s_model._num_subjs, empty_s_model._subjs, empty_s_model.data

        self.create_nodes()

        # loop over subjects
        for i_subj in range(n_subjs):
            # create and fit single subject
            if verbose > 1: print "*!*!* fitting subject %d *!*!*" % subjs[i_subj]
            t_data = self.data[self.data['subj_idx'] == subjs[i_subj]]
            t_data = rec_drop_fields(t_data, ['data_idx'])
            s_model = deepcopy(empty_s_model)
            s_model.data = t_data
            s_model.map(method='fmin_powell', runs=runs, **map_kwargs)

            # copy to original model
            for (name, node) in s_model.group_nodes.iteritems():
                self.subj_nodes[name][i_subj].value = node.value

        #set group and var nodes
        for (param_name, d) in self.params_dict.iteritems():
            for (tag, nodes) in d.subj_nodes.iteritems():
                subj_values = [x.value for x in nodes]
                #set group node
                if d.group_nodes:
                    d.group_nodes[tag].value = np.mean(subj_values)
                #set var node
                if d.var_nodes:
                    if d.var_type == 'std':
                        d.var_nodes[tag].value = np.std(subj_values)
                    elif d.var_type == 'precision':
                        d.var_nodes[tag].value = np.std(subj_values)**-2
                    elif d.var_type == 'sample_size':
                        v = np.var(subj_values)
                        m = np.mean(subj_values)
                        d.var_nodes[tag].value = (m * (1 - m)) / v - 1
                    else:
                        raise ValueError, "unknown var_type"




import kabuki
import kabuki.generate
import scipy.stats
from kabuki.hierarchical import *
import pymc as pm

# Build a very simple model

class TestModel(Hierarchical):
    def create_params(self):
        loc = pm.Uniform('loc', lower=-10, upper=10)
        var = pm.Uniform('var', lower=1e-8, upper=10)
        subj = pm.Normal('subj', mu=loc, tau=var)
        mu = HNode('mu', subj_knode=subj, group_knode=loc, var_knode=var)

        like_node = pm.Normal('like', mu=subj, tau=1)
        like = HNode('like', is_bottom_node=True, subj_knode=like_node)
        return [mu, like]

normal_like = kabuki.utils.scipy_stochastic(scipy.stats.distributions.norm_gen, name='normal', longname='normal')
data = kabuki.generate.gen_rand_data(normal_like, subjs=4, params={'mu': 0, 'tau':1})[0]

model = TestModel(data)
model.sample(20000)



from __future__ import division
import numpy as np
import pymc as pm
import re
from matplotlib.pylab import show, figure
import matplotlib.pyplot as plt
import sys
from operator import attrgetter

def convert_model_to_dictionary(model):
    """convert_model_to_dictionary(model)
    transform a set or list of nodes to a dictionary
    """
    d = {}
    for node in model:
        d[node.__name__] = node
    return d

def get_group_nodes(nodes, return_list=False):
    """
    get_group_nodes(model)
    get only the group nodes from the model
    """

    if type(nodes)==type({}):
        group_nodes = {}
        for name, node in nodes.iteritems():
            if (re.search('[A-Za-z)][0-9]+$',name) == None) and \
               not name.startswith('Metropolis') and \
               not name.startswith('deviance'):
                group_nodes[name] = node
        if return_list:
            return group_nodes.values()
        else:
            return group_nodes
    else:
        root = [z for z in nodes if re.search('[A-Za-z)][0-9]+$',z.__name__) == None]
        return root

def get_subjs_numbers(model):
    if isinstance(model, pm.MCMC):
        nodes = model.stochastics
    else:
        nodes = model

    s = [re.search('[0-9]+$',z.__name__) for z in nodes]
    return list(set([int(x) for x in s if x != None]))

def get_subj_nodes(model, startswith=None, i_subj=None):
    """get_subj_nodes(model, i_subj=None):
    return the nodes of subj i_subj. if is_subj is None then return all subjects' node
    if i_subj is -1, return root nodes
    """
    if isinstance(model, pm.MCMC):
        nodes = model.stochastics
    else:
        nodes = model

    if startswith is None:
        startswith = ''

    if i_subj==-1:
        return get_group_nodes(nodes)
    else:
        if type(nodes) == type({}):
            nodes = nodes.values()

        if i_subj is None:
            subj = [z for z in nodes if re.search(startswith+'[A-Za-z)][0-9]+$',z.__name__) != None]
        else:
            subj = [z for z in nodes if re.search(startswith+'[A-Za-z)]%d$'%i_subj,z.__name__) != None]

        if type(nodes) == type({}):
            return convert_model_to_dictionary(subj)
        else:
            return subj

def print_stats(stats):
    nodes = sorted(stats.keys());
    len_name = max([len(x) for x in nodes])
    f_names  = ['mean', 'std', '2.5q', '25q', '50q', '75q', '97.5', 'mc_err']
    len_f_names = 6

    s = 'name'.center(len_name) + '  '
    for name in f_names:
        s = s + ' ' + name.center(len_f_names)
    print s
    for node in nodes:
        v = stats[node]
        if type(v['mean']) == type(np.array([])) or node.startswith('Metropolis') or node == 'deviance':
            continue
        print "%s: %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f %6.3f" % \
        (node.ljust(len_name), v['mean'], v['standard deviation'],
         v['quantiles'][2.5], v['quantiles'][25],\
         v['quantiles'][50], v['quantiles'][75], \
         v['quantiles'][97.5], v['mc error'])

def print_group_stats(stats):
    g_stats = {}
    keys = [z for z in stats.keys() if re.match('[0-9]',z[-1]) is None]
    keys.sort()
    for key in keys:
        g_stats[key] = stats[key]
    print_stats(g_stats)

def group_plot(model, n_bins=50):
    if isinstance(model, pm.MCMC):
        nodes = model.stochastics
    else:
        nodes = model

    group_nodes = get_group_nodes(nodes, return_list=True)

    for node in group_nodes:
        pattern = ('%s[0-9]+'%node.__name__.replace("(","\(")).replace(')','\)')
        subj_nodes = [z for z in nodes if re.search(pattern,z.__name__) != None]
        if subj_nodes == []:
            continue

        print "plotting %s" % node.__name__
        sys.stdout.flush()
        figure()
        subj_nodes = sorted(subj_nodes, key=attrgetter('__name__'))
        lb = min([min(x.trace()) for x in subj_nodes])
        lb = min(lb, min(node.trace()))
        ub = max([max(x.trace()) for x in subj_nodes])
        ub = max(ub, max(node.trace()))
        x_data = np.linspace(lb, ub, n_bins)
        g_hist =np.histogram(node.trace(),bins=n_bins, range=[lb, ub], normed=True)[0]
        plt.plot(x_data, g_hist, '--', label='group')
        for i in subj_nodes:
            g_hist =np.histogram(i.trace(),bins=n_bins, range=[lb, ub], normed=True)[0]
            plt.plot(x_data, g_hist, label=re.search('[0-9]+$',i.__name__).group())
        plt.legend()
        plt.title(node.__name__)
    show()
